using CrystallographyBase: reciprocal, isreciprocal
using LinearAlgebra: I

@testset "Test `reciprocal` and `isreciprocal`" begin
    @testset "Base centered orthorhombic" begin
        a, b, c = 4, 3, 5
        lattice = Lattice([a, -b, 0] / 2, [a, b, 0] / 2, [0, 0, c])
        @test reciprocal(reciprocal(lattice)) == lattice
        @test isreciprocal(reciprocal(lattice), lattice)
        @test isreciprocal(lattice, reciprocal(lattice))
    end
    @testset "Body centered tetragonal" begin
        a, c = 4, 6
        lattice = Lattice([a, a, -c] / 2, [a, -a, c] / 2, [-a, a, c] / 2)
        @test reciprocal(reciprocal(lattice)) == lattice
        @test isreciprocal(reciprocal(lattice), lattice)
        @test isreciprocal(lattice, reciprocal(lattice))
    end
    @testset "Simple hexagonal" begin
        a, c = 2, 3.2
        lattice = Lattice([a, 0, 0], [a / 2, sqrt(3) / 2 * a, 0], [0, 0, c])
        @test isreciprocal(reciprocal(lattice), lattice)
        @test isreciprocal(lattice, reciprocal(lattice))
    end
    @testset "Face centered cubic" begin
        a = 4
        lattice = reciprocal(Lattice([
            1 1 0
            0 1 1
            1 0 1
        ]) * a / 2)
        @test reciprocal(reciprocal(lattice)) == lattice
        @test isreciprocal(reciprocal(lattice), lattice)
        @test isreciprocal(lattice, reciprocal(lattice))
    end
    @testset "Body centered cubic" begin
        a = 4
        lattice = Lattice([
            1 -1 1
            1 1 -1
            -1 1 1
        ]) * a / 2
        @test reciprocal(reciprocal(lattice)) == lattice
        @test isreciprocal(reciprocal(lattice), lattice)
        @test isreciprocal(lattice, reciprocal(lattice))
    end
end

# @testset "Test rutile structure" begin
#     lattice = [
#         4 0 0
#         0 4 0
#         0 0 3
#     ]
#     positions = [
#         0.0 0.5 0.3 0.7 0.2 0.8
#         0.0 0.5 0.3 0.7 0.8 0.2
#         0.0 0.5 0.0 0.0 0.5 0.5
#     ]
#     types = [14, 14, 8, 8, 8, 8]
#     rutile = Cell(lattice, positions, types)
#     alat = 4
#     @testset "No shift" begin
#         # The result has been compared with Python `spglib` and Quantum ESPRESSO.
#         @test coordinates(
#             reciprocal_mesh(rutile, [6, 6, 6]; ir_only = false, symprec = 1e-5),
#         ) ≈ [
#             [0.0, 0.0, 0.0],
#             [0.16666667, 0.0, 0.0],
#             [0.33333333, 0.0, 0.0],
#             [0.5, 0.0, 0.0],
#             [-0.33333333, 0.0, 0.0],
#             [-0.16666667, 0.0, 0.0],
#             [0.0, 0.16666667, 0.0],
#             [0.16666667, 0.16666667, 0.0],
#             [0.33333333, 0.16666667, 0.0],
#             [0.5, 0.16666667, 0.0],
#             [-0.33333333, 0.16666667, 0.0],
#             [-0.16666667, 0.16666667, 0.0],
#             [0.0, 0.33333333, 0.0],
#             [0.16666667, 0.33333333, 0.0],
#             [0.33333333, 0.33333333, 0.0],
#             [0.5, 0.33333333, 0.0],
#             [-0.33333333, 0.33333333, 0.0],
#             [-0.16666667, 0.33333333, 0.0],
#             [0.0, 0.5, 0.0],
#             [0.16666667, 0.5, 0.0],
#             [0.33333333, 0.5, 0.0],
#             [0.5, 0.5, 0.0],
#             [-0.33333333, 0.5, 0.0],
#             [-0.16666667, 0.5, 0.0],
#             [0.0, -0.33333333, 0.0],
#             [0.16666667, -0.33333333, 0.0],
#             [0.33333333, -0.33333333, 0.0],
#             [0.5, -0.33333333, 0.0],
#             [-0.33333333, -0.33333333, 0.0],
#             [-0.16666667, -0.33333333, 0.0],
#             [0.0, -0.16666667, 0.0],
#             [0.16666667, -0.16666667, 0.0],
#             [0.33333333, -0.16666667, 0.0],
#             [0.5, -0.16666667, 0.0],
#             [-0.33333333, -0.16666667, 0.0],
#             [-0.16666667, -0.16666667, 0.0],
#             [0.0, 0.0, 0.16666667],
#             [0.16666667, 0.0, 0.16666667],
#             [0.33333333, 0.0, 0.16666667],
#             [0.5, 0.0, 0.16666667],
#             [-0.33333333, 0.0, 0.16666667],
#             [-0.16666667, 0.0, 0.16666667],
#             [0.0, 0.16666667, 0.16666667],
#             [0.16666667, 0.16666667, 0.16666667],
#             [0.33333333, 0.16666667, 0.16666667],
#             [0.5, 0.16666667, 0.16666667],
#             [-0.33333333, 0.16666667, 0.16666667],
#             [-0.16666667, 0.16666667, 0.16666667],
#             [0.0, 0.33333333, 0.16666667],
#             [0.16666667, 0.33333333, 0.16666667],
#             [0.33333333, 0.33333333, 0.16666667],
#             [0.5, 0.33333333, 0.16666667],
#             [-0.33333333, 0.33333333, 0.16666667],
#             [-0.16666667, 0.33333333, 0.16666667],
#             [0.0, 0.5, 0.16666667],
#             [0.16666667, 0.5, 0.16666667],
#             [0.33333333, 0.5, 0.16666667],
#             [0.5, 0.5, 0.16666667],
#             [-0.33333333, 0.5, 0.16666667],
#             [-0.16666667, 0.5, 0.16666667],
#             [0.0, -0.33333333, 0.16666667],
#             [0.16666667, -0.33333333, 0.16666667],
#             [0.33333333, -0.33333333, 0.16666667],
#             [0.5, -0.33333333, 0.16666667],
#             [-0.33333333, -0.33333333, 0.16666667],
#             [-0.16666667, -0.33333333, 0.16666667],
#             [0.0, -0.16666667, 0.16666667],
#             [0.16666667, -0.16666667, 0.16666667],
#             [0.33333333, -0.16666667, 0.16666667],
#             [0.5, -0.16666667, 0.16666667],
#             [-0.33333333, -0.16666667, 0.16666667],
#             [-0.16666667, -0.16666667, 0.16666667],
#             [0.0, 0.0, 0.33333333],
#             [0.16666667, 0.0, 0.33333333],
#             [0.33333333, 0.0, 0.33333333],
#             [0.5, 0.0, 0.33333333],
#             [-0.33333333, 0.0, 0.33333333],
#             [-0.16666667, 0.0, 0.33333333],
#             [0.0, 0.16666667, 0.33333333],
#             [0.16666667, 0.16666667, 0.33333333],
#             [0.33333333, 0.16666667, 0.33333333],
#             [0.5, 0.16666667, 0.33333333],
#             [-0.33333333, 0.16666667, 0.33333333],
#             [-0.16666667, 0.16666667, 0.33333333],
#             [0.0, 0.33333333, 0.33333333],
#             [0.16666667, 0.33333333, 0.33333333],
#             [0.33333333, 0.33333333, 0.33333333],
#             [0.5, 0.33333333, 0.33333333],
#             [-0.33333333, 0.33333333, 0.33333333],
#             [-0.16666667, 0.33333333, 0.33333333],
#             [0.0, 0.5, 0.33333333],
#             [0.16666667, 0.5, 0.33333333],
#             [0.33333333, 0.5, 0.33333333],
#             [0.5, 0.5, 0.33333333],
#             [-0.33333333, 0.5, 0.33333333],
#             [-0.16666667, 0.5, 0.33333333],
#             [0.0, -0.33333333, 0.33333333],
#             [0.16666667, -0.33333333, 0.33333333],
#             [0.33333333, -0.33333333, 0.33333333],
#             [0.5, -0.33333333, 0.33333333],
#             [-0.33333333, -0.33333333, 0.33333333],
#             [-0.16666667, -0.33333333, 0.33333333],
#             [0.0, -0.16666667, 0.33333333],
#             [0.16666667, -0.16666667, 0.33333333],
#             [0.33333333, -0.16666667, 0.33333333],
#             [0.5, -0.16666667, 0.33333333],
#             [-0.33333333, -0.16666667, 0.33333333],
#             [-0.16666667, -0.16666667, 0.33333333],
#             [0.0, 0.0, 0.5],
#             [0.16666667, 0.0, 0.5],
#             [0.33333333, 0.0, 0.5],
#             [0.5, 0.0, 0.5],
#             [-0.33333333, 0.0, 0.5],
#             [-0.16666667, 0.0, 0.5],
#             [0.0, 0.16666667, 0.5],
#             [0.16666667, 0.16666667, 0.5],
#             [0.33333333, 0.16666667, 0.5],
#             [0.5, 0.16666667, 0.5],
#             [-0.33333333, 0.16666667, 0.5],
#             [-0.16666667, 0.16666667, 0.5],
#             [0.0, 0.33333333, 0.5],
#             [0.16666667, 0.33333333, 0.5],
#             [0.33333333, 0.33333333, 0.5],
#             [0.5, 0.33333333, 0.5],
#             [-0.33333333, 0.33333333, 0.5],
#             [-0.16666667, 0.33333333, 0.5],
#             [0.0, 0.5, 0.5],
#             [0.16666667, 0.5, 0.5],
#             [0.33333333, 0.5, 0.5],
#             [0.5, 0.5, 0.5],
#             [-0.33333333, 0.5, 0.5],
#             [-0.16666667, 0.5, 0.5],
#             [0.0, -0.33333333, 0.5],
#             [0.16666667, -0.33333333, 0.5],
#             [0.33333333, -0.33333333, 0.5],
#             [0.5, -0.33333333, 0.5],
#             [-0.33333333, -0.33333333, 0.5],
#             [-0.16666667, -0.33333333, 0.5],
#             [0.0, -0.16666667, 0.5],
#             [0.16666667, -0.16666667, 0.5],
#             [0.33333333, -0.16666667, 0.5],
#             [0.5, -0.16666667, 0.5],
#             [-0.33333333, -0.16666667, 0.5],
#             [-0.16666667, -0.16666667, 0.5],
#             [0.0, 0.0, -0.33333333],
#             [0.16666667, 0.0, -0.33333333],
#             [0.33333333, 0.0, -0.33333333],
#             [0.5, 0.0, -0.33333333],
#             [-0.33333333, 0.0, -0.33333333],
#             [-0.16666667, 0.0, -0.33333333],
#             [0.0, 0.16666667, -0.33333333],
#             [0.16666667, 0.16666667, -0.33333333],
#             [0.33333333, 0.16666667, -0.33333333],
#             [0.5, 0.16666667, -0.33333333],
#             [-0.33333333, 0.16666667, -0.33333333],
#             [-0.16666667, 0.16666667, -0.33333333],
#             [0.0, 0.33333333, -0.33333333],
#             [0.16666667, 0.33333333, -0.33333333],
#             [0.33333333, 0.33333333, -0.33333333],
#             [0.5, 0.33333333, -0.33333333],
#             [-0.33333333, 0.33333333, -0.33333333],
#             [-0.16666667, 0.33333333, -0.33333333],
#             [0.0, 0.5, -0.33333333],
#             [0.16666667, 0.5, -0.33333333],
#             [0.33333333, 0.5, -0.33333333],
#             [0.5, 0.5, -0.33333333],
#             [-0.33333333, 0.5, -0.33333333],
#             [-0.16666667, 0.5, -0.33333333],
#             [0.0, -0.33333333, -0.33333333],
#             [0.16666667, -0.33333333, -0.33333333],
#             [0.33333333, -0.33333333, -0.33333333],
#             [0.5, -0.33333333, -0.33333333],
#             [-0.33333333, -0.33333333, -0.33333333],
#             [-0.16666667, -0.33333333, -0.33333333],
#             [0.0, -0.16666667, -0.33333333],
#             [0.16666667, -0.16666667, -0.33333333],
#             [0.33333333, -0.16666667, -0.33333333],
#             [0.5, -0.16666667, -0.33333333],
#             [-0.33333333, -0.16666667, -0.33333333],
#             [-0.16666667, -0.16666667, -0.33333333],
#             [0.0, 0.0, -0.16666667],
#             [0.16666667, 0.0, -0.16666667],
#             [0.33333333, 0.0, -0.16666667],
#             [0.5, 0.0, -0.16666667],
#             [-0.33333333, 0.0, -0.16666667],
#             [-0.16666667, 0.0, -0.16666667],
#             [0.0, 0.16666667, -0.16666667],
#             [0.16666667, 0.16666667, -0.16666667],
#             [0.33333333, 0.16666667, -0.16666667],
#             [0.5, 0.16666667, -0.16666667],
#             [-0.33333333, 0.16666667, -0.16666667],
#             [-0.16666667, 0.16666667, -0.16666667],
#             [0.0, 0.33333333, -0.16666667],
#             [0.16666667, 0.33333333, -0.16666667],
#             [0.33333333, 0.33333333, -0.16666667],
#             [0.5, 0.33333333, -0.16666667],
#             [-0.33333333, 0.33333333, -0.16666667],
#             [-0.16666667, 0.33333333, -0.16666667],
#             [0.0, 0.5, -0.16666667],
#             [0.16666667, 0.5, -0.16666667],
#             [0.33333333, 0.5, -0.16666667],
#             [0.5, 0.5, -0.16666667],
#             [-0.33333333, 0.5, -0.16666667],
#             [-0.16666667, 0.5, -0.16666667],
#             [0.0, -0.33333333, -0.16666667],
#             [0.16666667, -0.33333333, -0.16666667],
#             [0.33333333, -0.33333333, -0.16666667],
#             [0.5, -0.33333333, -0.16666667],
#             [-0.33333333, -0.33333333, -0.16666667],
#             [-0.16666667, -0.33333333, -0.16666667],
#             [0.0, -0.16666667, -0.16666667],
#             [0.16666667, -0.16666667, -0.16666667],
#             [0.33333333, -0.16666667, -0.16666667],
#             [0.5, -0.16666667, -0.16666667],
#             [-0.33333333, -0.16666667, -0.16666667],
#             [-0.16666667, -0.16666667, -0.16666667],
#         ]
#         @test weights(reciprocal_mesh(rutile, [6, 6, 6]; ir_only = false, symprec = 1e-5)) ≈
#               fill(1 / 216, 216)
#         # The result has been compared with Quantum ESPRESSO.
#         # To compare with Quantum ESPRESSO's result, time your result with `alat`. See https://github.com/MineralsCloud/qe-demystify/blob/master/src/reciprocal%20axes.md
#         cartesian_benchmark = [
#             [0.0, 0.0, 0.0],
#             [0.2, 0.0, 0.0],
#             [0.4, 0.0, 0.0],
#             [-0.4, 0.0, 0.0],
#             [-0.2, 0.0, 0.0],
#             [0.0, 0.2, 0.0],
#             [0.2, 0.2, 0.0],
#             [0.4, 0.2, 0.0],
#             [-0.4, 0.2, 0.0],
#             [-0.2, 0.2, 0.0],
#             [0.0, 0.4, 0.0],
#             [0.2, 0.4, 0.0],
#             [0.4, 0.4, 0.0],
#             [-0.4, 0.4, 0.0],
#             [-0.2, 0.4, 0.0],
#             [0.0, -0.4, 0.0],
#             [0.2, -0.4, 0.0],
#             [0.4, -0.4, 0.0],
#             [-0.4, -0.4, 0.0],
#             [-0.2, -0.4, 0.0],
#             [0.0, -0.2, 0.0],
#             [0.2, -0.2, 0.0],
#             [0.4, -0.2, 0.0],
#             [-0.4, -0.2, 0.0],
#             [-0.2, -0.2, 0.0],
#             [0.0, 0.0, 0.4444444],
#             [0.2, 0.0, 0.4444444],
#             [0.4, 0.0, 0.4444444],
#             [-0.4, 0.0, 0.4444444],
#             [-0.2, 0.0, 0.4444444],
#             [0.0, 0.2, 0.4444444],
#             [0.2, 0.2, 0.4444444],
#             [0.4, 0.2, 0.4444444],
#             [-0.4, 0.2, 0.4444444],
#             [-0.2, 0.2, 0.4444444],
#             [0.0, 0.4, 0.4444444],
#             [0.2, 0.4, 0.4444444],
#             [0.4, 0.4, 0.4444444],
#             [-0.4, 0.4, 0.4444444],
#             [-0.2, 0.4, 0.4444444],
#             [0.0, -0.4, 0.4444444],
#             [0.2, -0.4, 0.4444444],
#             [0.4, -0.4, 0.4444444],
#             [-0.4, -0.4, 0.4444444],
#             [-0.2, -0.4, 0.4444444],
#             [0.0, -0.2, 0.4444444],
#             [0.2, -0.2, 0.4444444],
#             [0.4, -0.2, 0.4444444],
#             [-0.4, -0.2, 0.4444444],
#             [-0.2, -0.2, 0.4444444],
#             [0.0, 0.0, -0.4444444],
#             [0.2, 0.0, -0.4444444],
#             [0.4, 0.0, -0.4444444],
#             [-0.4, 0.0, -0.4444444],
#             [-0.2, 0.0, -0.4444444],
#             [0.0, 0.2, -0.4444444],
#             [0.2, 0.2, -0.4444444],
#             [0.4, 0.2, -0.4444444],
#             [-0.4, 0.2, -0.4444444],
#             [-0.2, 0.2, -0.4444444],
#             [0.0, 0.4, -0.4444444],
#             [0.2, 0.4, -0.4444444],
#             [0.4, 0.4, -0.4444444],
#             [-0.4, 0.4, -0.4444444],
#             [-0.2, 0.4, -0.4444444],
#             [0.0, -0.4, -0.4444444],
#             [0.2, -0.4, -0.4444444],
#             [0.4, -0.4, -0.4444444],
#             [-0.4, -0.4, -0.4444444],
#             [-0.2, -0.4, -0.4444444],
#             [0.0, -0.2, -0.4444444],
#             [0.2, -0.2, -0.4444444],
#             [0.4, -0.2, -0.4444444],
#             [-0.4, -0.2, -0.4444444],
#             [-0.2, -0.2, -0.4444444],
#         ]
#         fractional =
#             coordinates(reciprocal_mesh(rutile, [5, 5, 3]; ir_only = false, symprec = 1e-5))
#         cartesian = coordinates(
#             reciprocal_mesh(
#                 rutile,
#                 [5, 5, 3];
#                 ir_only = false,
#                 symprec = 1e-5,
#                 cartesian = true,
#             ),
#         )
#         @test isapprox(cartesian * alat, cartesian_benchmark; atol = 1e-6)
#         @test fractional == FractionalFromCartesian(reciprocal(Lattice(rutile))).(cartesian)  # Test `FractionalFromCartesian`
#     end
#     @testset "With shift" begin
#         @test isapprox(
#             coordinates(
#                 reciprocal_mesh(
#                     rutile,
#                     [4, 4, 4],
#                     trues(3);
#                     ir_only = false,
#                     symprec = 1e-5,
#                     cartesian = true,
#                 ),
#             ) * alat,
#             1;
#             atol = 1e-6,
#         )
#     end
#     # The result has been compared with Python `spglib` and Quantum ESPRESSO.
#     @test coordinates(
#         reciprocal_mesh(rutile, [5, 5, 5], trues(3); ir_only = false, symprec = 1e-5),
#     ) == [
#         [0.1, 0.1, 0.1],
#         [0.3, 0.1, 0.1],
#         [0.5, 0.1, 0.1],
#         [-0.3, 0.1, 0.1],
#         [-0.1, 0.1, 0.1],
#         [0.1, 0.3, 0.1],
#         [0.3, 0.3, 0.1],
#         [0.5, 0.3, 0.1],
#         [-0.3, 0.3, 0.1],
#         [-0.1, 0.3, 0.1],
#         [0.1, 0.5, 0.1],
#         [0.3, 0.5, 0.1],
#         [0.5, 0.5, 0.1],
#         [-0.3, 0.5, 0.1],
#         [-0.1, 0.5, 0.1],
#         [0.1, -0.3, 0.1],
#         [0.3, -0.3, 0.1],
#         [0.5, -0.3, 0.1],
#         [-0.3, -0.3, 0.1],
#         [-0.1, -0.3, 0.1],
#         [0.1, -0.1, 0.1],
#         [0.3, -0.1, 0.1],
#         [0.5, -0.1, 0.1],
#         [-0.3, -0.1, 0.1],
#         [-0.1, -0.1, 0.1],
#         [0.1, 0.1, 0.3],
#         [0.3, 0.1, 0.3],
#         [0.5, 0.1, 0.3],
#         [-0.3, 0.1, 0.3],
#         [-0.1, 0.1, 0.3],
#         [0.1, 0.3, 0.3],
#         [0.3, 0.3, 0.3],
#         [0.5, 0.3, 0.3],
#         [-0.3, 0.3, 0.3],
#         [-0.1, 0.3, 0.3],
#         [0.1, 0.5, 0.3],
#         [0.3, 0.5, 0.3],
#         [0.5, 0.5, 0.3],
#         [-0.3, 0.5, 0.3],
#         [-0.1, 0.5, 0.3],
#         [0.1, -0.3, 0.3],
#         [0.3, -0.3, 0.3],
#         [0.5, -0.3, 0.3],
#         [-0.3, -0.3, 0.3],
#         [-0.1, -0.3, 0.3],
#         [0.1, -0.1, 0.3],
#         [0.3, -0.1, 0.3],
#         [0.5, -0.1, 0.3],
#         [-0.3, -0.1, 0.3],
#         [-0.1, -0.1, 0.3],
#         [0.1, 0.1, 0.5],
#         [0.3, 0.1, 0.5],
#         [0.5, 0.1, 0.5],
#         [-0.3, 0.1, 0.5],
#         [-0.1, 0.1, 0.5],
#         [0.1, 0.3, 0.5],
#         [0.3, 0.3, 0.5],
#         [0.5, 0.3, 0.5],
#         [-0.3, 0.3, 0.5],
#         [-0.1, 0.3, 0.5],
#         [0.1, 0.5, 0.5],
#         [0.3, 0.5, 0.5],
#         [0.5, 0.5, 0.5],
#         [-0.3, 0.5, 0.5],
#         [-0.1, 0.5, 0.5],
#         [0.1, -0.3, 0.5],
#         [0.3, -0.3, 0.5],
#         [0.5, -0.3, 0.5],
#         [-0.3, -0.3, 0.5],
#         [-0.1, -0.3, 0.5],
#         [0.1, -0.1, 0.5],
#         [0.3, -0.1, 0.5],
#         [0.5, -0.1, 0.5],
#         [-0.3, -0.1, 0.5],
#         [-0.1, -0.1, 0.5],
#         [0.1, 0.1, -0.3],
#         [0.3, 0.1, -0.3],
#         [0.5, 0.1, -0.3],
#         [-0.3, 0.1, -0.3],
#         [-0.1, 0.1, -0.3],
#         [0.1, 0.3, -0.3],
#         [0.3, 0.3, -0.3],
#         [0.5, 0.3, -0.3],
#         [-0.3, 0.3, -0.3],
#         [-0.1, 0.3, -0.3],
#         [0.1, 0.5, -0.3],
#         [0.3, 0.5, -0.3],
#         [0.5, 0.5, -0.3],
#         [-0.3, 0.5, -0.3],
#         [-0.1, 0.5, -0.3],
#         [0.1, -0.3, -0.3],
#         [0.3, -0.3, -0.3],
#         [0.5, -0.3, -0.3],
#         [-0.3, -0.3, -0.3],
#         [-0.1, -0.3, -0.3],
#         [0.1, -0.1, -0.3],
#         [0.3, -0.1, -0.3],
#         [0.5, -0.1, -0.3],
#         [-0.3, -0.1, -0.3],
#         [-0.1, -0.1, -0.3],
#         [0.1, 0.1, -0.1],
#         [0.3, 0.1, -0.1],
#         [0.5, 0.1, -0.1],
#         [-0.3, 0.1, -0.1],
#         [-0.1, 0.1, -0.1],
#         [0.1, 0.3, -0.1],
#         [0.3, 0.3, -0.1],
#         [0.5, 0.3, -0.1],
#         [-0.3, 0.3, -0.1],
#         [-0.1, 0.3, -0.1],
#         [0.1, 0.5, -0.1],
#         [0.3, 0.5, -0.1],
#         [0.5, 0.5, -0.1],
#         [-0.3, 0.5, -0.1],
#         [-0.1, 0.5, -0.1],
#         [0.1, -0.3, -0.1],
#         [0.3, -0.3, -0.1],
#         [0.5, -0.3, -0.1],
#         [-0.3, -0.3, -0.1],
#         [-0.1, -0.3, -0.1],
#         [0.1, -0.1, -0.1],
#         [0.3, -0.1, -0.1],
#         [0.5, -0.1, -0.1],
#         [-0.3, -0.1, -0.1],
#         [-0.1, -0.1, -0.1],
#     ]
# end

@testset "Test simple rutile structure" begin
    lattice = [[0, 5.4, 5.4], [5.4, 0, 5.4], [5.4, 5.4, 0]]
    positions = [[0, 0, 0], [0.25, 0.25, 0.25]]
    types = [1, 1]
    cell = Cell(lattice, positions, types)
    # Compared with Quantum ESPRESSO and VASP: https://github.com/spglib/spglib/issues/79#issue-462738630
    # @test coordinates(reciprocal_mesh(cell, [3, 3, 3], [1, 1, 1]; symprec = 1e-5)) == [
    #     [0.16666666666666666, 0.16666666666666666, 0.16666666666666666],
    #     [0.5, 0.16666666666666666, 0.16666666666666666],
    #     [-0.16666666666666666, 0.16666666666666666, 0.16666666666666666],
    #     [0.5, 0.5, 0.16666666666666666],
    #     [-0.16666666666666666, 0.5, 0.16666666666666666],
    #     [0.5, 0.5, 0.5],
    # ]
    # @test weights(reciprocal_mesh(cell, [3, 3, 3], [1, 1, 1], symprec = 1e-5)) ==
    #       [96, 288, 288, 288, 288, 48] / 1296
end

# @testset "`spglib` doc example" begin  # See https://spglib.github.io/spglib/python-spglib.html#get-ir-reciprocal-mesh
#     lattice = [
#         0 0.5 0.5
#         0.5 0 0.5
#         0.5 0.5 0
#     ] * 5.4
#     positions = [[0.875, 0.875, 0.875], [0.125, 0.125, 0.125]]
#     types = [1, 1]
#     cell = Cell(lattice, positions, types)
#     @testset "No shift" begin
#         # The result has been compared with Python `spglib` and Quantum ESPRESSO.
#         @test coordinates(
#             reciprocal_mesh(cell, [4, 4, 4]; ir_only = false, symprec = 1e-5),
#         ) == [
#             [0.0, 0.0, 0.0],
#             [0.25, 0.0, 0.0],
#             [0.5, 0.0, 0.0],
#             [-0.25, 0.0, 0.0],
#             [0.0, 0.25, 0.0],
#             [0.25, 0.25, 0.0],
#             [0.5, 0.25, 0.0],
#             [-0.25, 0.25, 0.0],
#             [0.0, 0.5, 0.0],
#             [0.25, 0.5, 0.0],
#             [0.5, 0.5, 0.0],
#             [-0.25, 0.5, 0.0],
#             [0.0, -0.25, 0.0],
#             [0.25, -0.25, 0.0],
#             [0.5, -0.25, 0.0],
#             [-0.25, -0.25, 0.0],
#             [0.0, 0.0, 0.25],
#             [0.25, 0.0, 0.25],
#             [0.5, 0.0, 0.25],
#             [-0.25, 0.0, 0.25],
#             [0.0, 0.25, 0.25],
#             [0.25, 0.25, 0.25],
#             [0.5, 0.25, 0.25],
#             [-0.25, 0.25, 0.25],
#             [0.0, 0.5, 0.25],
#             [0.25, 0.5, 0.25],
#             [0.5, 0.5, 0.25],
#             [-0.25, 0.5, 0.25],
#             [0.0, -0.25, 0.25],
#             [0.25, -0.25, 0.25],
#             [0.5, -0.25, 0.25],
#             [-0.25, -0.25, 0.25],
#             [0.0, 0.0, 0.5],
#             [0.25, 0.0, 0.5],
#             [0.5, 0.0, 0.5],
#             [-0.25, 0.0, 0.5],
#             [0.0, 0.25, 0.5],
#             [0.25, 0.25, 0.5],
#             [0.5, 0.25, 0.5],
#             [-0.25, 0.25, 0.5],
#             [0.0, 0.5, 0.5],
#             [0.25, 0.5, 0.5],
#             [0.5, 0.5, 0.5],
#             [-0.25, 0.5, 0.5],
#             [0.0, -0.25, 0.5],
#             [0.25, -0.25, 0.5],
#             [0.5, -0.25, 0.5],
#             [-0.25, -0.25, 0.5],
#             [0.0, 0.0, -0.25],
#             [0.25, 0.0, -0.25],
#             [0.5, 0.0, -0.25],
#             [-0.25, 0.0, -0.25],
#             [0.0, 0.25, -0.25],
#             [0.25, 0.25, -0.25],
#             [0.5, 0.25, -0.25],
#             [-0.25, 0.25, -0.25],
#             [0.0, 0.5, -0.25],
#             [0.25, 0.5, -0.25],
#             [0.5, 0.5, -0.25],
#             [-0.25, 0.5, -0.25],
#             [0.0, -0.25, -0.25],
#             [0.25, -0.25, -0.25],
#             [0.5, -0.25, -0.25],
#             [-0.25, -0.25, -0.25],
#         ]
#         @test weights(reciprocal_mesh(cell, [4, 4, 4]; ir_only = false, symprec = 1e-5)) ==
#               fill(1 / 64, 64)
#         # The result has been compared with Python `spglib` and Quantum ESPRESSO.
#         @test coordinates(
#             reciprocal_mesh(cell, [5, 5, 5]; ir_only = false, symprec = 1e-5),
#         ) ≈ [
#             [0.0, 0.0, 0.0],
#             [0.2, 0.0, -0.0],
#             [0.4, 0.0, -0.0],
#             [-0.4, -0.0, 0.0],
#             [-0.2, -0.0, 0.0],
#             [0.0, 0.2, 0.0],
#             [0.2, 0.2, 0.0],
#             [0.4, 0.2, 0.0],
#             [-0.4, 0.2, 0.0],
#             [-0.2, 0.2, -0.0],
#             [0.0, 0.4, 0.0],
#             [0.2, 0.4, 0.0],
#             [0.4, 0.4, 0.0],
#             [-0.4, 0.4, 0.0],
#             [-0.2, 0.4, 0.0],
#             [-0.0, -0.4, -0.0],
#             [0.2, -0.4, -0.0],
#             [0.4, -0.4, -0.0],
#             [-0.4, -0.4, -0.0],
#             [-0.2, -0.4, -0.0],
#             [-0.0, -0.2, -0.0],
#             [0.2, -0.2, 0.0],
#             [0.4, -0.2, 0.0],
#             [-0.4, -0.2, -0.0],
#             [-0.2, -0.2, 0.0],
#             [-0.0, -0.0, 0.2],
#             [0.2, 0.0, 0.2],
#             [0.4, 0.0, 0.2],
#             [-0.4, -0.0, 0.2],
#             [-0.2, 0.0, 0.2],
#             [0.0, 0.2, 0.2],
#             [0.2, 0.2, 0.2],
#             [0.4, 0.2, 0.2],
#             [-0.4, 0.2, 0.2],
#             [-0.2, 0.2, 0.2],
#             [0.0, 0.4, 0.2],
#             [0.2, 0.4, 0.2],
#             [0.4, 0.4, 0.2],
#             [-0.4, 0.4, 0.2],
#             [-0.2, 0.4, 0.2],
#             [-0.0, -0.4, 0.2],
#             [0.2, -0.4, 0.2],
#             [0.4, -0.4, 0.2],
#             [-0.4, -0.4, 0.2],
#             [-0.2, -0.4, 0.2],
#             [0.0, -0.2, 0.2],
#             [0.2, -0.2, 0.2],
#             [0.4, -0.2, 0.2],
#             [-0.4, -0.2, 0.2],
#             [-0.2, -0.2, 0.2],
#             [-0.0, -0.0, 0.4],
#             [0.2, 0.0, 0.4],
#             [0.4, 0.0, 0.4],
#             [-0.4, -0.0, 0.4],
#             [-0.2, 0.0, 0.4],
#             [0.0, 0.2, 0.4],
#             [0.2, 0.2, 0.4],
#             [0.4, 0.2, 0.4],
#             [-0.4, 0.2, 0.4],
#             [-0.2, 0.2, 0.4],
#             [0.0, 0.4, 0.4],
#             [0.2, 0.4, 0.4],
#             [0.4, 0.4, 0.4],
#             [-0.4, 0.4, 0.4],
#             [-0.2, 0.4, 0.4],
#             [-0.0, -0.4, 0.4],
#             [0.2, -0.4, 0.4],
#             [0.4, -0.4, 0.4],
#             [-0.4, -0.4, 0.4],
#             [-0.2, -0.4, 0.4],
#             [0.0, -0.2, 0.4],
#             [0.2, -0.2, 0.4],
#             [0.4, -0.2, 0.4],
#             [-0.4, -0.2, 0.4],
#             [-0.2, -0.2, 0.4],
#             [0.0, 0.0, -0.4],
#             [0.2, 0.0, -0.4],
#             [0.4, 0.0, -0.4],
#             [-0.4, -0.0, -0.4],
#             [-0.2, -0.0, -0.4],
#             [0.0, 0.2, -0.4],
#             [0.2, 0.2, -0.4],
#             [0.4, 0.2, -0.4],
#             [-0.4, 0.2, -0.4],
#             [-0.2, 0.2, -0.4],
#             [0.0, 0.4, -0.4],
#             [0.2, 0.4, -0.4],
#             [0.4, 0.4, -0.4],
#             [-0.4, 0.4, -0.4],
#             [-0.2, 0.4, -0.4],
#             [-0.0, -0.4, -0.4],
#             [0.2, -0.4, -0.4],
#             [0.4, -0.4, -0.4],
#             [-0.4, -0.4, -0.4],
#             [-0.2, -0.4, -0.4],
#             [-0.0, -0.2, -0.4],
#             [0.2, -0.2, -0.4],
#             [0.4, -0.2, -0.4],
#             [-0.4, -0.2, -0.4],
#             [-0.2, -0.2, -0.4],
#             [0.0, 0.0, -0.2],
#             [0.2, -0.0, -0.2],
#             [0.4, 0.0, -0.2],
#             [-0.4, -0.0, -0.2],
#             [-0.2, 0.0, -0.2],
#             [-0.0, 0.2, -0.2],
#             [0.2, 0.2, -0.2],
#             [0.4, 0.2, -0.2],
#             [-0.4, 0.2, -0.2],
#             [-0.2, 0.2, -0.2],
#             [0.0, 0.4, -0.2],
#             [0.2, 0.4, -0.2],
#             [0.4, 0.4, -0.2],
#             [-0.4, 0.4, -0.2],
#             [-0.2, 0.4, -0.2],
#             [-0.0, -0.4, -0.2],
#             [0.2, -0.4, -0.2],
#             [0.4, -0.4, -0.2],
#             [-0.4, -0.4, -0.2],
#             [-0.2, -0.4, -0.2],
#             [0.0, -0.2, -0.2],
#             [0.2, -0.2, -0.2],
#             [0.4, -0.2, -0.2],
#             [-0.4, -0.2, -0.2],
#             [-0.2, -0.2, -0.2],
#         ]
#         # The result has been compared with Quantum ESPRESSO.
#         @test isapprox(
#             coordinates(
#                 reciprocal_mesh(
#                     cell,
#                     [5, 5, 5];
#                     ir_only = false,
#                     symprec = 1e-5,
#                     cartesian = true,
#                 ),
#             ) *
#             5.4 *
#             0.5 *
#             sqrt(2),  # Quantum ESPRESSO's result is tricky
#             [
#                 [0.0, 0.0, 0.0],
#                 [-0.1414214, 0.1414214, 0.1414214],
#                 [-0.2828427, 0.2828427, 0.2828427],
#                 [0.2828427, -0.2828427, -0.2828427],
#                 [0.1414214, -0.1414214, -0.1414214],
#                 [0.1414214, -0.1414214, 0.1414214],
#                 [0.0, 0.0, 0.2828427],
#                 [-0.1414214, 0.1414214, 0.4242641],
#                 [0.4242641, -0.4242641, -0.1414214],
#                 [0.2828427, -0.2828427, 0.0],
#                 [0.2828427, -0.2828427, 0.2828427],
#                 [0.1414214, -0.1414214, 0.4242641],
#                 [0.0, 0.0, 0.5656854],
#                 [0.5656854, -0.5656854, 0.0],
#                 [0.4242641, -0.4242641, 0.1414214],
#                 [-0.2828427, 0.2828427, -0.2828427],
#                 [-0.4242641, 0.4242641, -0.1414214],
#                 [-0.5656854, 0.5656854, -0.0],
#                 [-0.0, -0.0, -0.5656854],
#                 [-0.1414214, 0.1414214, -0.4242641],
#                 [-0.1414214, 0.1414214, -0.1414214],
#                 [-0.2828427, 0.2828427, 0.0],
#                 [-0.4242641, 0.4242641, 0.1414214],
#                 [0.1414214, -0.1414214, -0.4242641],
#                 [0.0, 0.0, -0.2828427],
#                 [0.1414214, 0.1414214, -0.1414214],
#                 [-0.0, 0.2828427, 0.0],
#                 [-0.1414214, 0.4242641, 0.1414214],
#                 [0.4242641, -0.1414214, -0.4242641],
#                 [0.2828427, 0.0, -0.2828427],
#                 [0.2828427, -0.0, 0.0],
#                 [0.1414214, 0.1414214, 0.1414214],
#                 [0.0, 0.2828427, 0.2828427],
#                 [0.5656854, -0.2828427, -0.2828427],
#                 [0.4242641, -0.1414214, -0.1414214],
#                 [0.4242641, -0.1414214, 0.1414214],
#                 [0.2828427, 0.0, 0.2828427],
#                 [0.1414214, 0.1414214, 0.4242641],
#                 [0.7071068, -0.4242641, -0.1414214],
#                 [0.5656854, -0.2828427, 0.0],
#                 [-0.1414214, 0.4242641, -0.4242641],
#                 [-0.2828427, 0.5656854, -0.2828427],
#                 [-0.4242641, 0.7071068, -0.1414214],
#                 [0.1414214, 0.1414214, -0.7071068],
#                 [-0.0, 0.2828427, -0.5656854],
#                 [0.0, 0.2828427, -0.2828427],
#                 [-0.1414214, 0.4242641, -0.1414214],
#                 [-0.2828427, 0.5656854, 0.0],
#                 [0.2828427, -0.0, -0.5656854],
#                 [0.1414214, 0.1414214, -0.4242641],
#                 [0.2828427, 0.2828427, -0.2828427],
#                 [0.1414214, 0.4242641, -0.1414214],
#                 [-0.0, 0.5656854, 0.0],
#                 [0.5656854, -0.0, -0.5656854],
#                 [0.4242641, 0.1414214, -0.4242641],
#                 [0.4242641, 0.1414214, -0.1414214],
#                 [0.2828427, 0.2828427, 0.0],
#                 [0.1414214, 0.4242641, 0.1414214],
#                 [0.7071068, -0.1414214, -0.4242641],
#                 [0.5656854, 0.0, -0.2828427],
#                 [0.5656854, -0.0, 0.0],
#                 [0.4242641, 0.1414214, 0.1414214],
#                 [0.2828427, 0.2828427, 0.2828427],
#                 [0.8485281, -0.2828427, -0.2828427],
#                 [0.7071068, -0.1414214, -0.1414214],
#                 [-0.0, 0.5656854, -0.5656854],
#                 [-0.1414214, 0.7071068, -0.4242641],
#                 [-0.2828427, 0.8485281, -0.2828427],
#                 [0.2828427, 0.2828427, -0.8485281],
#                 [0.1414214, 0.4242641, -0.7071068],
#                 [0.1414214, 0.4242641, -0.4242641],
#                 [0.0, 0.5656854, -0.2828427],
#                 [-0.1414214, 0.7071068, -0.1414214],
#                 [0.4242641, 0.1414214, -0.7071068],
#                 [0.2828427, 0.2828427, -0.5656854],
#                 [-0.2828427, -0.2828427, 0.2828427],
#                 [-0.4242641, -0.1414214, 0.4242641],
#                 [-0.5656854, 0.0, 0.5656854],
#                 [0.0, -0.5656854, -0.0],
#                 [-0.1414214, -0.4242641, 0.1414214],
#                 [-0.1414214, -0.4242641, 0.4242641],
#                 [-0.2828427, -0.2828427, 0.5656854],
#                 [-0.4242641, -0.1414214, 0.7071068],
#                 [0.1414214, -0.7071068, 0.1414214],
#                 [-0.0, -0.5656854, 0.2828427],
#                 [0.0, -0.5656854, 0.5656854],
#                 [-0.1414214, -0.4242641, 0.7071068],
#                 [-0.2828427, -0.2828427, 0.8485281],
#                 [0.2828427, -0.8485281, 0.2828427],
#                 [0.1414214, -0.7071068, 0.4242641],
#                 [-0.5656854, 0.0, -0.0],
#                 [-0.7071068, 0.1414214, 0.1414214],
#                 [-0.8485281, 0.2828427, 0.2828427],
#                 [-0.2828427, -0.2828427, -0.2828427],
#                 [-0.4242641, -0.1414214, -0.1414214],
#                 [-0.4242641, -0.1414214, 0.1414214],
#                 [-0.5656854, -0.0, 0.2828427],
#                 [-0.7071068, 0.1414214, 0.4242641],
#                 [-0.1414214, -0.4242641, -0.1414214],
#                 [-0.2828427, -0.2828427, 0.0],
#                 [-0.1414214, -0.1414214, 0.1414214],
#                 [-0.2828427, 0.0, 0.2828427],
#                 [-0.4242641, 0.1414214, 0.4242641],
#                 [0.1414214, -0.4242641, -0.1414214],
#                 [-0.0, -0.2828427, 0.0],
#                 [0.0, -0.2828427, 0.2828427],
#                 [-0.1414214, -0.1414214, 0.4242641],
#                 [-0.2828427, 0.0, 0.5656854],
#                 [0.2828427, -0.5656854, -0.0],
#                 [0.1414214, -0.4242641, 0.1414214],
#                 [0.1414214, -0.4242641, 0.4242641],
#                 [0.0, -0.2828427, 0.5656854],
#                 [-0.1414214, -0.1414214, 0.7071068],
#                 [0.4242641, -0.7071068, 0.1414214],
#                 [0.2828427, -0.5656854, 0.2828427],
#                 [-0.4242641, 0.1414214, -0.1414214],
#                 [-0.5656854, 0.2828427, -0.0],
#                 [-0.7071068, 0.4242641, 0.1414214],
#                 [-0.1414214, -0.1414214, -0.4242641],
#                 [-0.2828427, 0.0, -0.2828427],
#                 [-0.2828427, -0.0, 0.0],
#                 [-0.4242641, 0.1414214, 0.1414214],
#                 [-0.5656854, 0.2828427, 0.2828427],
#                 [0.0, -0.2828427, -0.2828427],
#                 [-0.1414214, -0.1414214, -0.1414214],
#             ];
#             atol = 1e-6,
#         )
#     end
#     @testset "With shift" begin
#         # Partly verified with Quantum ESPRESSO
#         @test coordinates(
#                   reciprocal_mesh(
#                       cell,
#                       [5, 5, 5],
#                       trues(3);
#                       ir_only = false,
#                       symprec = 1e-5,
#                       cartesian = true,
#                   ),
#               ) *
#               5.4 *
#               0.5 *
#               sqrt(2) ≈ [
#             [0.07071067811865477, 0.07071067811865477, 0.07071067811865477],
#             [-0.07071067811865477, 0.21213203435596426, 0.21213203435596426],
#             [-0.21213203435596426, 0.3535533905932738, 0.3535533905932738],
#             [0.3535533905932738, -0.21213203435596426, -0.21213203435596426],
#             [0.21213203435596426, -0.07071067811865477, -0.07071067811865477],
#             [0.21213203435596426, -0.07071067811865477, 0.21213203435596426],
#             [0.07071067811865477, 0.07071067811865477, 0.3535533905932738],
#             [-0.07071067811865477, 0.21213203435596426, 0.4949747468305833],
#             [0.4949747468305833, -0.3535533905932738, -0.07071067811865477],
#             [0.3535533905932738, -0.21213203435596426, 0.07071067811865477],
#             [0.3535533905932738, -0.21213203435596426, 0.3535533905932738],
#             [0.21213203435596426, -0.07071067811865477, 0.4949747468305833],
#             [0.07071067811865477, 0.07071067811865477, 0.6363961030678928],
#             [0.6363961030678928, -0.4949747468305833, 0.07071067811865477],
#             [0.4949747468305833, -0.3535533905932738, 0.21213203435596426],
#             [-0.21213203435596426, 0.3535533905932738, -0.21213203435596426],
#             [-0.3535533905932738, 0.4949747468305833, -0.07071067811865477],
#             [-0.4949747468305833, 0.6363961030678928, 0.07071067811865477],
#             [0.07071067811865477, 0.07071067811865477, -0.4949747468305833],
#             [-0.07071067811865477, 0.21213203435596426, -0.3535533905932738],
#             [-0.07071067811865477, 0.21213203435596426, -0.07071067811865477],
#             [-0.21213203435596426, 0.3535533905932738, 0.07071067811865477],
#             [-0.3535533905932738, 0.4949747468305833, 0.21213203435596426],
#             [0.21213203435596426, -0.07071067811865477, -0.3535533905932738],
#             [0.07071067811865477, 0.07071067811865477, -0.21213203435596426],
#             [0.21213203435596426, 0.21213203435596426, -0.07071067811865474],
#             [0.07071067811865474, 0.3535533905932738, 0.07071067811865477],
#             [-0.07071067811865477, 0.4949747468305833, 0.21213203435596426],
#             [0.4949747468305833, -0.07071067811865477, -0.3535533905932738],
#             [0.3535533905932738, 0.07071067811865474, -0.21213203435596426],
#             [0.3535533905932738, 0.07071067811865474, 0.07071067811865477],
#             [0.21213203435596426, 0.21213203435596426, 0.21213203435596426],
#             [0.07071067811865474, 0.3535533905932738, 0.3535533905932738],
#             [0.6363961030678928, -0.21213203435596426, -0.21213203435596426],
#             [0.4949747468305833, -0.07071067811865477, -0.07071067811865474],
#             [0.4949747468305833, -0.07071067811865477, 0.21213203435596426],
#             [0.3535533905932738, 0.07071067811865474, 0.3535533905932738],
#             [0.21213203435596426, 0.21213203435596426, 0.4949747468305833],
#             [0.7778174593052024, -0.3535533905932738, -0.07071067811865474],
#             [0.6363961030678928, -0.21213203435596426, 0.07071067811865477],
#             [-0.07071067811865477, 0.4949747468305833, -0.3535533905932738],
#             [-0.21213203435596426, 0.6363961030678928, -0.21213203435596426],
#             [-0.3535533905932738, 0.7778174593052024, -0.07071067811865474],
#             [0.21213203435596426, 0.21213203435596426, -0.6363961030678928],
#             [0.07071067811865474, 0.3535533905932738, -0.4949747468305833],
#             [0.07071067811865474, 0.3535533905932738, -0.21213203435596426],
#             [-0.07071067811865477, 0.4949747468305833, -0.07071067811865474],
#             [-0.21213203435596426, 0.6363961030678928, 0.07071067811865477],
#             [0.3535533905932738, 0.07071067811865474, -0.4949747468305833],
#             [0.21213203435596426, 0.21213203435596426, -0.3535533905932738],
#             [0.3535533905932738, 0.3535533905932738, -0.21213203435596426],
#             [0.21213203435596426, 0.4949747468305833, -0.07071067811865477],
#             [0.07071067811865477, 0.6363961030678928, 0.07071067811865477],
#             [0.6363961030678928, 0.07071067811865477, -0.4949747468305833],
#             [0.4949747468305833, 0.21213203435596426, -0.3535533905932738],
#             [0.4949747468305833, 0.21213203435596426, -0.07071067811865477],
#             [0.3535533905932738, 0.3535533905932738, 0.07071067811865477],
#             [0.21213203435596426, 0.4949747468305833, 0.21213203435596426],
#             [0.7778174593052024, -0.07071067811865477, -0.3535533905932738],
#             [0.6363961030678928, 0.07071067811865477, -0.21213203435596426],
#             [0.6363961030678928, 0.07071067811865477, 0.07071067811865477],
#             [0.4949747468305833, 0.21213203435596426, 0.21213203435596426],
#             [0.3535533905932738, 0.3535533905932738, 0.3535533905932738],
#             [0.9192388155425119, -0.21213203435596426, -0.21213203435596426],
#             [0.7778174593052024, -0.07071067811865477, -0.07071067811865477],
#             [0.07071067811865477, 0.6363961030678928, -0.4949747468305833],
#             [-0.07071067811865477, 0.7778174593052024, -0.3535533905932738],
#             [-0.21213203435596426, 0.9192388155425119, -0.21213203435596426],
#             [0.3535533905932738, 0.3535533905932738, -0.7778174593052024],
#             [0.21213203435596426, 0.4949747468305833, -0.6363961030678928],
#             [0.21213203435596426, 0.4949747468305833, -0.3535533905932738],
#             [0.07071067811865477, 0.6363961030678928, -0.21213203435596426],
#             [-0.07071067811865477, 0.7778174593052024, -0.07071067811865477],
#             [0.4949747468305833, 0.21213203435596426, -0.6363961030678928],
#             [0.3535533905932738, 0.3535533905932738, -0.4949747468305833],
#             [-0.21213203435596426, -0.21213203435596426, 0.3535533905932738],
#             [-0.3535533905932738, -0.07071067811865474, 0.4949747468305833],
#             [-0.4949747468305833, 0.07071067811865477, 0.6363961030678928],
#             [0.07071067811865477, -0.4949747468305833, 0.07071067811865474],
#             [-0.07071067811865474, -0.3535533905932738, 0.21213203435596426],
#             [-0.07071067811865474, -0.3535533905932738, 0.4949747468305833],
#             [-0.21213203435596426, -0.21213203435596426, 0.6363961030678928],
#             [-0.3535533905932738, -0.07071067811865474, 0.7778174593052024],
#             [0.21213203435596426, -0.6363961030678928, 0.21213203435596426],
#             [0.07071067811865477, -0.4949747468305833, 0.3535533905932738],
#             [0.07071067811865477, -0.4949747468305833, 0.6363961030678928],
#             [-0.07071067811865474, -0.3535533905932738, 0.7778174593052024],
#             [-0.21213203435596426, -0.21213203435596426, 0.9192388155425119],
#             [0.3535533905932738, -0.7778174593052024, 0.3535533905932738],
#             [0.21213203435596426, -0.6363961030678928, 0.4949747468305833],
#             [-0.4949747468305833, 0.07071067811865477, 0.07071067811865474],
#             [-0.6363961030678928, 0.21213203435596426, 0.21213203435596426],
#             [-0.7778174593052024, 0.3535533905932738, 0.3535533905932738],
#             [-0.21213203435596426, -0.21213203435596426, -0.21213203435596426],
#             [-0.3535533905932738, -0.07071067811865474, -0.07071067811865477],
#             [-0.3535533905932738, -0.07071067811865474, 0.21213203435596426],
#             [-0.4949747468305833, 0.07071067811865477, 0.3535533905932738],
#             [-0.6363961030678928, 0.21213203435596426, 0.4949747468305833],
#             [-0.07071067811865474, -0.3535533905932738, -0.07071067811865477],
#             [-0.21213203435596426, -0.21213203435596426, 0.07071067811865474],
#             [-0.07071067811865477, -0.07071067811865477, 0.21213203435596426],
#             [-0.21213203435596426, 0.07071067811865477, 0.3535533905932738],
#             [-0.3535533905932738, 0.21213203435596426, 0.4949747468305833],
#             [0.21213203435596426, -0.3535533905932738, -0.07071067811865477],
#             [0.07071067811865477, -0.21213203435596426, 0.07071067811865477],
#             [0.07071067811865477, -0.21213203435596426, 0.3535533905932738],
#             [-0.07071067811865477, -0.07071067811865477, 0.4949747468305833],
#             [-0.21213203435596426, 0.07071067811865477, 0.6363961030678928],
#             [0.3535533905932738, -0.4949747468305833, 0.07071067811865477],
#             [0.21213203435596426, -0.3535533905932738, 0.21213203435596426],
#             [0.21213203435596426, -0.3535533905932738, 0.4949747468305833],
#             [0.07071067811865477, -0.21213203435596426, 0.6363961030678928],
#             [-0.07071067811865477, -0.07071067811865477, 0.7778174593052024],
#             [0.4949747468305833, -0.6363961030678928, 0.21213203435596426],
#             [0.3535533905932738, -0.4949747468305833, 0.3535533905932738],
#             [-0.3535533905932738, 0.21213203435596426, -0.07071067811865477],
#             [-0.4949747468305833, 0.3535533905932738, 0.07071067811865477],
#             [-0.6363961030678928, 0.4949747468305833, 0.21213203435596426],
#             [-0.07071067811865477, -0.07071067811865477, -0.3535533905932738],
#             [-0.21213203435596426, 0.07071067811865477, -0.21213203435596426],
#             [-0.21213203435596426, 0.07071067811865477, 0.07071067811865477],
#             [-0.3535533905932738, 0.21213203435596426, 0.21213203435596426],
#             [-0.4949747468305833, 0.3535533905932738, 0.3535533905932738],
#             [0.07071067811865477, -0.21213203435596426, -0.21213203435596426],
#             [-0.07071067811865477, -0.07071067811865477, -0.07071067811865477],
#         ]
#     end
# end

# See https://github.com/JuliaMolSim/DFTK.jl/blob/a444606/test/testcases.jl#L48-L92
@testset "Test Mg structure" begin
    lattice = [
        -3.0179389205999998 -3.0179389205999998 0.0000000000000000
        -5.2272235447000002 5.2272235447000002 0.0000000000000000
        0.0000000000000000 0.0000000000000000 -9.7736219469000005
    ]
    positions = [[2 / 3, 1 / 3, 1 / 4], [1 / 3, 2 / 3, 3 / 4]]
    types = [1, 1]
    cell = Cell(lattice, positions, types)
    # @test coordinates(reciprocal_mesh(cell, [3, 3, 3]; symprec = 1e-5)) == [
    #     [0, 0, 0],
    #     [1 / 3, 0, 0],
    #     [1 / 3, 1 / 3, 0],
    #     [0, 0, 1 / 3],
    #     [1 / 3, 0, 1 / 3],
    #     [1 / 3, 1 / 3, 1 / 3],
    # ]
end

# @testset "Test MgB₂ structure" begin
#     a = 3.07
#     c = 3.52
#     lattice = [
#         a 0 0
#         -a/2 a/2*sqrt(3) 0
#         0 0 c
#     ]
#     positions = [
#         0 0 0
#         1.0/3 2.0/3 0.5
#         2.0/3 1.0/3 0.5
#     ]
#     types = [12, 5, 5]
#     MgB₂ = Cell(lattice, positions, types)
#     @test reciprocal_mesh(MgB₂, [7, 7, 7]; ir_only = false, cartesian = true) - [
#         [0.0, 0.0, 0.0, 1.0],
#         [0.1428571, 0.0824786, 0.0, 1.0],
#         [0.2857143, 0.1649572, 0.0, 1.0],
#         [0.4285714, 0.2474358, 0.0, 1.0],
#         [0.5714286, 0.3299144, 0.0, 1.0],
#         [0.7142857, 0.412393, 0.0, 1.0],
#         [0.8571429, 0.4948717, 0.0, 1.0],
#         [0.0, 0.1649572, 0.0, 1.0],
#         [0.1428571, 0.2474358, 0.0, 1.0],
#         [0.2857143, 0.3299144, 0.0, 1.0],
#         [0.4285714, 0.412393, 0.0, 1.0],
#         [0.5714286, 0.4948717, 0.0, 1.0],
#         [0.7142857, 0.5773503, 0.0, 1.0],
#         [0.8571429, 0.6598289, 0.0, 1.0],
#         [0.0, 0.3299144, 0.0, 1.0],
#         [0.1428571, 0.412393, 0.0, 1.0],
#         [0.2857143, 0.4948717, 0.0, 1.0],
#         [0.4285714, 0.5773503, 0.0, 1.0],
#         [0.5714286, 0.6598289, 0.0, 1.0],
#         [0.7142857, 0.7423075, 0.0, 1.0],
#         [0.8571429, 0.8247861, 0.0, 1.0],
#         [0.0, 0.4948717, 0.0, 1.0],
#         [0.1428571, 0.5773503, 0.0, 1.0],
#         [0.2857143, 0.6598289, 0.0, 1.0],
#         [0.4285714, 0.7423075, 0.0, 1.0],
#         [0.5714286, 0.8247861, 0.0, 1.0],
#         [0.7142857, 0.9072647, 0.0, 1.0],
#         [0.8571429, 0.9897433, 0.0, 1.0],
#         [0.0, 0.6598289, 0.0, 1.0],
#         [0.1428571, 0.7423075, 0.0, 1.0],
#         [0.2857143, 0.8247861, 0.0, 1.0],
#         [0.4285714, 0.9072647, 0.0, 1.0],
#         [0.5714286, 0.9897433, 0.0, 1.0],
#         [0.7142857, 1.0722219, 0.0, 1.0],
#         [0.8571429, 1.1547005, 0.0, 1.0],
#         [0.0, 0.8247861, 0.0, 1.0],
#         [0.1428571, 0.9072647, 0.0, 1.0],
#         [0.2857143, 0.9897433, 0.0, 1.0],
#         [0.4285714, 1.0722219, 0.0, 1.0],
#         [0.5714286, 1.1547005, 0.0, 1.0],
#         [0.7142857, 1.2371791, 0.0, 1.0],
#         [0.8571429, 1.3196578, 0.0, 1.0],
#         [0.0, 0.9897433, 0.0, 1.0],
#         [0.1428571, 1.0722219, 0.0, 1.0],
#         [0.2857143, 1.1547005, 0.0, 1.0],
#         [0.4285714, 1.2371791, 0.0, 1.0],
#         [0.5714286, 1.3196578, 0.0, 1.0],
#         [0.7142857, 1.4021364, 0.0, 1.0],
#         [0.8571429, 1.484615, 0.0, 1.0],
#         [0.0, 0.0, 0.1245942, 1.0],
#         [0.1428571, 0.0824786, 0.1245942, 1.0],
#         [0.2857143, 0.1649572, 0.1245942, 1.0],
#         [0.4285714, 0.2474358, 0.1245942, 1.0],
#         [0.5714286, 0.3299144, 0.1245942, 1.0],
#         [0.7142857, 0.412393, 0.1245942, 1.0],
#         [0.8571429, 0.4948717, 0.1245942, 1.0],
#         [0.0, 0.1649572, 0.1245942, 1.0],
#         [0.1428571, 0.2474358, 0.1245942, 1.0],
#         [0.2857143, 0.3299144, 0.1245942, 1.0],
#         [0.4285714, 0.412393, 0.1245942, 1.0],
#         [0.5714286, 0.4948717, 0.1245942, 1.0],
#         [0.7142857, 0.5773503, 0.1245942, 1.0],
#         [0.8571429, 0.6598289, 0.1245942, 1.0],
#         [0.0, 0.3299144, 0.1245942, 1.0],
#         [0.1428571, 0.412393, 0.1245942, 1.0],
#         [0.2857143, 0.4948717, 0.1245942, 1.0],
#         [0.4285714, 0.5773503, 0.1245942, 1.0],
#         [0.5714286, 0.6598289, 0.1245942, 1.0],
#         [0.7142857, 0.7423075, 0.1245942, 1.0],
#         [0.8571429, 0.8247861, 0.1245942, 1.0],
#         [0.0, 0.4948717, 0.1245942, 1.0],
#         [0.1428571, 0.5773503, 0.1245942, 1.0],
#         [0.2857143, 0.6598289, 0.1245942, 1.0],
#         [0.4285714, 0.7423075, 0.1245942, 1.0],
#         [0.5714286, 0.8247861, 0.1245942, 1.0],
#         [0.7142857, 0.9072647, 0.1245942, 1.0],
#         [0.8571429, 0.9897433, 0.1245942, 1.0],
#         [0.0, 0.6598289, 0.1245942, 1.0],
#         [0.1428571, 0.7423075, 0.1245942, 1.0],
#         [0.2857143, 0.8247861, 0.1245942, 1.0],
#         [0.4285714, 0.9072647, 0.1245942, 1.0],
#         [0.5714286, 0.9897433, 0.1245942, 1.0],
#         [0.7142857, 1.0722219, 0.1245942, 1.0],
#         [0.8571429, 1.1547005, 0.1245942, 1.0],
#         [0.0, 0.8247861, 0.1245942, 1.0],
#         [0.1428571, 0.9072647, 0.1245942, 1.0],
#         [0.2857143, 0.9897433, 0.1245942, 1.0],
#         [0.4285714, 1.0722219, 0.1245942, 1.0],
#         [0.5714286, 1.1547005, 0.1245942, 1.0],
#         [0.7142857, 1.2371791, 0.1245942, 1.0],
#         [0.8571429, 1.3196578, 0.1245942, 1.0],
#         [0.0, 0.9897433, 0.1245942, 1.0],
#         [0.1428571, 1.0722219, 0.1245942, 1.0],
#         [0.2857143, 1.1547005, 0.1245942, 1.0],
#         [0.4285714, 1.2371791, 0.1245942, 1.0],
#         [0.5714286, 1.3196578, 0.1245942, 1.0],
#         [0.7142857, 1.4021364, 0.1245942, 1.0],
#         [0.8571429, 1.484615, 0.1245942, 1.0],
#         [0.0, 0.0, 0.2491883, 1.0],
#         [0.1428571, 0.0824786, 0.2491883, 1.0],
#         [0.2857143, 0.1649572, 0.2491883, 1.0],
#         [0.4285714, 0.2474358, 0.2491883, 1.0],
#         [0.5714286, 0.3299144, 0.2491883, 1.0],
#         [0.7142857, 0.412393, 0.2491883, 1.0],
#         [0.8571429, 0.4948717, 0.2491883, 1.0],
#         [0.0, 0.1649572, 0.2491883, 1.0],
#         [0.1428571, 0.2474358, 0.2491883, 1.0],
#         [0.2857143, 0.3299144, 0.2491883, 1.0],
#         [0.4285714, 0.412393, 0.2491883, 1.0],
#         [0.5714286, 0.4948717, 0.2491883, 1.0],
#         [0.7142857, 0.5773503, 0.2491883, 1.0],
#         [0.8571429, 0.6598289, 0.2491883, 1.0],
#         [0.0, 0.3299144, 0.2491883, 1.0],
#         [0.1428571, 0.412393, 0.2491883, 1.0],
#         [0.2857143, 0.4948717, 0.2491883, 1.0],
#         [0.4285714, 0.5773503, 0.2491883, 1.0],
#         [0.5714286, 0.6598289, 0.2491883, 1.0],
#         [0.7142857, 0.7423075, 0.2491883, 1.0],
#         [0.8571429, 0.8247861, 0.2491883, 1.0],
#         [0.0, 0.4948717, 0.2491883, 1.0],
#         [0.1428571, 0.5773503, 0.2491883, 1.0],
#         [0.2857143, 0.6598289, 0.2491883, 1.0],
#         [0.4285714, 0.7423075, 0.2491883, 1.0],
#         [0.5714286, 0.8247861, 0.2491883, 1.0],
#         [0.7142857, 0.9072647, 0.2491883, 1.0],
#         [0.8571429, 0.9897433, 0.2491883, 1.0],
#         [0.0, 0.6598289, 0.2491883, 1.0],
#         [0.1428571, 0.7423075, 0.2491883, 1.0],
#         [0.2857143, 0.8247861, 0.2491883, 1.0],
#         [0.4285714, 0.9072647, 0.2491883, 1.0],
#         [0.5714286, 0.9897433, 0.2491883, 1.0],
#         [0.7142857, 1.0722219, 0.2491883, 1.0],
#         [0.8571429, 1.1547005, 0.2491883, 1.0],
#         [0.0, 0.8247861, 0.2491883, 1.0],
#         [0.1428571, 0.9072647, 0.2491883, 1.0],
#         [0.2857143, 0.9897433, 0.2491883, 1.0],
#         [0.4285714, 1.0722219, 0.2491883, 1.0],
#         [0.5714286, 1.1547005, 0.2491883, 1.0],
#         [0.7142857, 1.2371791, 0.2491883, 1.0],
#         [0.8571429, 1.3196578, 0.2491883, 1.0],
#         [0.0, 0.9897433, 0.2491883, 1.0],
#         [0.1428571, 1.0722219, 0.2491883, 1.0],
#         [0.2857143, 1.1547005, 0.2491883, 1.0],
#         [0.4285714, 1.2371791, 0.2491883, 1.0],
#         [0.5714286, 1.3196578, 0.2491883, 1.0],
#         [0.7142857, 1.4021364, 0.2491883, 1.0],
#         [0.8571429, 1.484615, 0.2491883, 1.0],
#         [0.0, 0.0, 0.3737825, 1.0],
#         [0.1428571, 0.0824786, 0.3737825, 1.0],
#         [0.2857143, 0.1649572, 0.3737825, 1.0],
#         [0.4285714, 0.2474358, 0.3737825, 1.0],
#         [0.5714286, 0.3299144, 0.3737825, 1.0],
#         [0.7142857, 0.412393, 0.3737825, 1.0],
#         [0.8571429, 0.4948717, 0.3737825, 1.0],
#         [0.0, 0.1649572, 0.3737825, 1.0],
#         [0.1428571, 0.2474358, 0.3737825, 1.0],
#         [0.2857143, 0.3299144, 0.3737825, 1.0],
#         [0.4285714, 0.412393, 0.3737825, 1.0],
#         [0.5714286, 0.4948717, 0.3737825, 1.0],
#         [0.7142857, 0.5773503, 0.3737825, 1.0],
#         [0.8571429, 0.6598289, 0.3737825, 1.0],
#         [0.0, 0.3299144, 0.3737825, 1.0],
#         [0.1428571, 0.412393, 0.3737825, 1.0],
#         [0.2857143, 0.4948717, 0.3737825, 1.0],
#         [0.4285714, 0.5773503, 0.3737825, 1.0],
#         [0.5714286, 0.6598289, 0.3737825, 1.0],
#         [0.7142857, 0.7423075, 0.3737825, 1.0],
#         [0.8571429, 0.8247861, 0.3737825, 1.0],
#         [0.0, 0.4948717, 0.3737825, 1.0],
#         [0.1428571, 0.5773503, 0.3737825, 1.0],
#         [0.2857143, 0.6598289, 0.3737825, 1.0],
#         [0.4285714, 0.7423075, 0.3737825, 1.0],
#         [0.5714286, 0.8247861, 0.3737825, 1.0],
#         [0.7142857, 0.9072647, 0.3737825, 1.0],
#         [0.8571429, 0.9897433, 0.3737825, 1.0],
#         [0.0, 0.6598289, 0.3737825, 1.0],
#         [0.1428571, 0.7423075, 0.3737825, 1.0],
#         [0.2857143, 0.8247861, 0.3737825, 1.0],
#         [0.4285714, 0.9072647, 0.3737825, 1.0],
#         [0.5714286, 0.9897433, 0.3737825, 1.0],
#         [0.7142857, 1.0722219, 0.3737825, 1.0],
#         [0.8571429, 1.1547005, 0.3737825, 1.0],
#         [0.0, 0.8247861, 0.3737825, 1.0],
#         [0.1428571, 0.9072647, 0.3737825, 1.0],
#         [0.2857143, 0.9897433, 0.3737825, 1.0],
#         [0.4285714, 1.0722219, 0.3737825, 1.0],
#         [0.5714286, 1.1547005, 0.3737825, 1.0],
#         [0.7142857, 1.2371791, 0.3737825, 1.0],
#         [0.8571429, 1.3196578, 0.3737825, 1.0],
#         [0.0, 0.9897433, 0.3737825, 1.0],
#         [0.1428571, 1.0722219, 0.3737825, 1.0],
#         [0.2857143, 1.1547005, 0.3737825, 1.0],
#         [0.4285714, 1.2371791, 0.3737825, 1.0],
#         [0.5714286, 1.3196578, 0.3737825, 1.0],
#         [0.7142857, 1.4021364, 0.3737825, 1.0],
#         [0.8571429, 1.484615, 0.3737825, 1.0],
#         [0.0, 0.0, 0.4983766, 1.0],
#         [0.1428571, 0.0824786, 0.4983766, 1.0],
#         [0.2857143, 0.1649572, 0.4983766, 1.0],
#         [0.4285714, 0.2474358, 0.4983766, 1.0],
#         [0.5714286, 0.3299144, 0.4983766, 1.0],
#         [0.7142857, 0.412393, 0.4983766, 1.0],
#         [0.8571429, 0.4948717, 0.4983766, 1.0],
#         [0.0, 0.1649572, 0.4983766, 1.0],
#         [0.1428571, 0.2474358, 0.4983766, 1.0],
#         [0.2857143, 0.3299144, 0.4983766, 1.0],
#         [0.4285714, 0.412393, 0.4983766, 1.0],
#         [0.5714286, 0.4948717, 0.4983766, 1.0],
#         [0.7142857, 0.5773503, 0.4983766, 1.0],
#         [0.8571429, 0.6598289, 0.4983766, 1.0],
#         [0.0, 0.3299144, 0.4983766, 1.0],
#         [0.1428571, 0.412393, 0.4983766, 1.0],
#         [0.2857143, 0.4948717, 0.4983766, 1.0],
#         [0.4285714, 0.5773503, 0.4983766, 1.0],
#         [0.5714286, 0.6598289, 0.4983766, 1.0],
#         [0.7142857, 0.7423075, 0.4983766, 1.0],
#         [0.8571429, 0.8247861, 0.4983766, 1.0],
#         [0.0, 0.4948717, 0.4983766, 1.0],
#         [0.1428571, 0.5773503, 0.4983766, 1.0],
#         [0.2857143, 0.6598289, 0.4983766, 1.0],
#         [0.4285714, 0.7423075, 0.4983766, 1.0],
#         [0.5714286, 0.8247861, 0.4983766, 1.0],
#         [0.7142857, 0.9072647, 0.4983766, 1.0],
#         [0.8571429, 0.9897433, 0.4983766, 1.0],
#         [0.0, 0.6598289, 0.4983766, 1.0],
#         [0.1428571, 0.7423075, 0.4983766, 1.0],
#         [0.2857143, 0.8247861, 0.4983766, 1.0],
#         [0.4285714, 0.9072647, 0.4983766, 1.0],
#         [0.5714286, 0.9897433, 0.4983766, 1.0],
#         [0.7142857, 1.0722219, 0.4983766, 1.0],
#         [0.8571429, 1.1547005, 0.4983766, 1.0],
#         [0.0, 0.8247861, 0.4983766, 1.0],
#         [0.1428571, 0.9072647, 0.4983766, 1.0],
#         [0.2857143, 0.9897433, 0.4983766, 1.0],
#         [0.4285714, 1.0722219, 0.4983766, 1.0],
#         [0.5714286, 1.1547005, 0.4983766, 1.0],
#         [0.7142857, 1.2371791, 0.4983766, 1.0],
#         [0.8571429, 1.3196578, 0.4983766, 1.0],
#         [0.0, 0.9897433, 0.4983766, 1.0],
#         [0.1428571, 1.0722219, 0.4983766, 1.0],
#         [0.2857143, 1.1547005, 0.4983766, 1.0],
#         [0.4285714, 1.2371791, 0.4983766, 1.0],
#         [0.5714286, 1.3196578, 0.4983766, 1.0],
#         [0.7142857, 1.4021364, 0.4983766, 1.0],
#         [0.8571429, 1.484615, 0.4983766, 1.0],
#         [0.0, 0.0, 0.6229708, 1.0],
#         [0.1428571, 0.0824786, 0.6229708, 1.0],
#         [0.2857143, 0.1649572, 0.6229708, 1.0],
#         [0.4285714, 0.2474358, 0.6229708, 1.0],
#         [0.5714286, 0.3299144, 0.6229708, 1.0],
#         [0.7142857, 0.412393, 0.6229708, 1.0],
#         [0.8571429, 0.4948717, 0.6229708, 1.0],
#         [0.0, 0.1649572, 0.6229708, 1.0],
#         [0.1428571, 0.2474358, 0.6229708, 1.0],
#         [0.2857143, 0.3299144, 0.6229708, 1.0],
#         [0.4285714, 0.412393, 0.6229708, 1.0],
#         [0.5714286, 0.4948717, 0.6229708, 1.0],
#         [0.7142857, 0.5773503, 0.6229708, 1.0],
#         [0.8571429, 0.6598289, 0.6229708, 1.0],
#         [0.0, 0.3299144, 0.6229708, 1.0],
#         [0.1428571, 0.412393, 0.6229708, 1.0],
#         [0.2857143, 0.4948717, 0.6229708, 1.0],
#         [0.4285714, 0.5773503, 0.6229708, 1.0],
#         [0.5714286, 0.6598289, 0.6229708, 1.0],
#         [0.7142857, 0.7423075, 0.6229708, 1.0],
#         [0.8571429, 0.8247861, 0.6229708, 1.0],
#         [0.0, 0.4948717, 0.6229708, 1.0],
#         [0.1428571, 0.5773503, 0.6229708, 1.0],
#         [0.2857143, 0.6598289, 0.6229708, 1.0],
#         [0.4285714, 0.7423075, 0.6229708, 1.0],
#         [0.5714286, 0.8247861, 0.6229708, 1.0],
#         [0.7142857, 0.9072647, 0.6229708, 1.0],
#         [0.8571429, 0.9897433, 0.6229708, 1.0],
#         [0.0, 0.6598289, 0.6229708, 1.0],
#         [0.1428571, 0.7423075, 0.6229708, 1.0],
#         [0.2857143, 0.8247861, 0.6229708, 1.0],
#         [0.4285714, 0.9072647, 0.6229708, 1.0],
#         [0.5714286, 0.9897433, 0.6229708, 1.0],
#         [0.7142857, 1.0722219, 0.6229708, 1.0],
#         [0.8571429, 1.1547005, 0.6229708, 1.0],
#         [0.0, 0.8247861, 0.6229708, 1.0],
#         [0.1428571, 0.9072647, 0.6229708, 1.0],
#         [0.2857143, 0.9897433, 0.6229708, 1.0],
#         [0.4285714, 1.0722219, 0.6229708, 1.0],
#         [0.5714286, 1.1547005, 0.6229708, 1.0],
#         [0.7142857, 1.2371791, 0.6229708, 1.0],
#         [0.8571429, 1.3196578, 0.6229708, 1.0],
#         [0.0, 0.9897433, 0.6229708, 1.0],
#         [0.1428571, 1.0722219, 0.6229708, 1.0],
#         [0.2857143, 1.1547005, 0.6229708, 1.0],
#         [0.4285714, 1.2371791, 0.6229708, 1.0],
#         [0.5714286, 1.3196578, 0.6229708, 1.0],
#         [0.7142857, 1.4021364, 0.6229708, 1.0],
#         [0.8571429, 1.484615, 0.6229708, 1.0],
#         [0.0, 0.0, 0.7475649, 1.0],
#         [0.1428571, 0.0824786, 0.7475649, 1.0],
#         [0.2857143, 0.1649572, 0.7475649, 1.0],
#         [0.4285714, 0.2474358, 0.7475649, 1.0],
#         [0.5714286, 0.3299144, 0.7475649, 1.0],
#         [0.7142857, 0.412393, 0.7475649, 1.0],
#         [0.8571429, 0.4948717, 0.7475649, 1.0],
#         [0.0, 0.1649572, 0.7475649, 1.0],
#         [0.1428571, 0.2474358, 0.7475649, 1.0],
#         [0.2857143, 0.3299144, 0.7475649, 1.0],
#         [0.4285714, 0.412393, 0.7475649, 1.0],
#         [0.5714286, 0.4948717, 0.7475649, 1.0],
#         [0.7142857, 0.5773503, 0.7475649, 1.0],
#         [0.8571429, 0.6598289, 0.7475649, 1.0],
#         [0.0, 0.3299144, 0.7475649, 1.0],
#         [0.1428571, 0.412393, 0.7475649, 1.0],
#         [0.2857143, 0.4948717, 0.7475649, 1.0],
#         [0.4285714, 0.5773503, 0.7475649, 1.0],
#         [0.5714286, 0.6598289, 0.7475649, 1.0],
#         [0.7142857, 0.7423075, 0.7475649, 1.0],
#         [0.8571429, 0.8247861, 0.7475649, 1.0],
#         [0.0, 0.4948717, 0.7475649, 1.0],
#         [0.1428571, 0.5773503, 0.7475649, 1.0],
#         [0.2857143, 0.6598289, 0.7475649, 1.0],
#         [0.4285714, 0.7423075, 0.7475649, 1.0],
#         [0.5714286, 0.8247861, 0.7475649, 1.0],
#         [0.7142857, 0.9072647, 0.7475649, 1.0],
#         [0.8571429, 0.9897433, 0.7475649, 1.0],
#         [0.0, 0.6598289, 0.7475649, 1.0],
#         [0.1428571, 0.7423075, 0.7475649, 1.0],
#         [0.2857143, 0.8247861, 0.7475649, 1.0],
#         [0.4285714, 0.9072647, 0.7475649, 1.0],
#         [0.5714286, 0.9897433, 0.7475649, 1.0],
#         [0.7142857, 1.0722219, 0.7475649, 1.0],
#         [0.8571429, 1.1547005, 0.7475649, 1.0],
#         [0.0, 0.8247861, 0.7475649, 1.0],
#         [0.1428571, 0.9072647, 0.7475649, 1.0],
#         [0.2857143, 0.9897433, 0.7475649, 1.0],
#         [0.4285714, 1.0722219, 0.7475649, 1.0],
#         [0.5714286, 1.1547005, 0.7475649, 1.0],
#         [0.7142857, 1.2371791, 0.7475649, 1.0],
#         [0.8571429, 1.3196578, 0.7475649, 1.0],
#         [0.0, 0.9897433, 0.7475649, 1.0],
#         [0.1428571, 1.0722219, 0.7475649, 1.0],
#         [0.2857143, 1.1547005, 0.7475649, 1.0],
#         [0.4285714, 1.2371791, 0.7475649, 1.0],
#         [0.5714286, 1.3196578, 0.7475649, 1.0],
#         [0.7142857, 1.4021364, 0.7475649, 1.0],
#         [0.8571429, 1.484615, 0.7475649, 1.0],
#     ]
# end

@testset "Construct `MonkhorstPackGrid` incorrectly" begin
    @test_throws DimensionMismatch MonkhorstPackGrid([4, 4, 4, 4], [1, 1, 1])
    @test_throws AssertionError MonkhorstPackGrid([4, 4, 0], [1, 1, 1])
    @test_throws DimensionMismatch MonkhorstPackGrid([4, 4, 4], [1, 1, 1, 1])
    @test_throws InexactError MonkhorstPackGrid([4, 4, 4], [1, 1, 2])
end

# @testset "Test `ReciprocalPath`" begin
#     lattice = Lattice([0, 1, 1], [1, 0, 1], [1, 1, 0]) / 2
#     path = ReciprocalPath(lattice, 227)
#     # Compared with https://thchr.github.io/Brillouin.jl/dev/kpaths/
#     @test path.special_points == Dict(
#         :U => [0.625, 0.25, 0.625],
#         :W => [0.5, 0.25, 0.75],
#         :K => [0.375, 0.375, 0.75],
#         :Γ => [0.0, 0.0, 0.0],
#         :L => [0.5, 0.5, 0.5],
#         :X => [0.5, 0.0, 0.5],
#     )
#     @test path.suggested_paths == [[:Γ, :X, :U], [:K, :Γ, :L, :W, :X]]
#     @test cartesianize(irrfbz_path(227, 2pi * [[1, 0, 0], [0, 1, 0], [0, 0, 1]])).points ==
#           coordinates(path, true) ==
#           Dict(
#               :U => [0.25, 1.0, 0.25],
#               :W => [0.5, 1.0, 0.0],
#               :K => [0.75, 0.75, 0.0],
#               :Γ => [0.0, 0.0, 0.0],
#               :L => [0.5, 0.5, 0.5],
#               :X => [0.0, 1.0, 0.0],
#           )
# end
